######################################################################
#
# Macro Name    : a_conversion_kml_to_str.tcl
#
# Created by    : Ash Colton from Dassault Systemes.
#
# Created on    : July 2017
#
# Description   : Will take a kml file with lat long and convert it to a string file with UTM coordinates and the zone number written into the d4 field.
#
# Conditions of use : Macro may be used and distributed freely, however may not be modified from it's original version without first contacting Ash Colton on 08 9420 1384.
#                     If modifications or improvements are made to this macro, please email the modified macro to ash.colton@3ds.com
#
######################################################################

set convertKMLtoSTR {
  GuidoForm form {
    -label "Convert KML to STR"
    -default_buttons
    -defaults_key "MACRO_kml_to_str"
    -layout BoxLayout Y_AXIS
    
    GuidoPanel panel {
      -label "<html><b>File to Convert</b>"
      -border etched true

      GuidoFileBrowserField kmlFile {
        -label "Select KML file"
        -width 25
        -null false
        -extension false
        -file_mask *.kml
        -absolutePath true
      }
    }
    GuidoButtonGroupPanel grid {
      -label "<html><b>Select Grid</b>"
      -border etched true
      GuidoRadioButton mga {
        -caption MGA
        -selected_value mga
        -tip "UTM North has a false northing of 0m and a false easting of 500000m whereas UTM South has a false northing of 10000000m and a false easting of 500000m. Both UTM North and South use WGS 84 as their spheroid."
      }
#       GuidoRadioButton amg {
#         -caption AMG
#         -selected_value amg
#         -tip "UTM North has a false northing of 0m and a false easting of 500000m whereas UTM South has a false northing of 10000000m and a false easting of 500000m. Both UTM North and South use WGS 84 as their spheroid."
#       }
      GuidoRadioButton utmNorth {
        -caption "North of Equator"
        -selected_value utmNorth
        -tip "UTM North has a false northing of 0m and a false easting of 500000m whereas UTM South has a false northing of 10000000m and a false easting of 500000m. Both UTM North and South use WGS 84 as their spheroid."
      }
      GuidoRadioButton utmSouth {
        -caption "South of Equator"
        -selected_value utmSouth
        -tip "UTM North has a false northing of 0m and a false easting of 500000m whereas UTM South has a false northing of 10000000m and a false easting of 500000m. Both UTM North and South use WGS 84 as their spheroid."
      }
    }
    GuidoPanel panel2 {
      -label "<html><b>For North and South of the Equator selections</b>"
      -border etched true
      -layout CentreLineLayout vertical left

      GuidoCheckBox negativeLongitude {
        -label "Negative Longitude?"
        -caption "Yes"
        -selected_value "yes"
        -unselected_value "no"
        -default "no"
        -dependency {"[$grid getCurrentValue]" == "utmNorth" || "[$grid getCurrentValue]" == "utmSouth"}
      }
      GuidoLabel label1 {
        -label "<html><font color=red>Any zone between 1 and 30</font> will have <font color=red>negative</font> longitude."
      }
      GuidoFiller fill1 {
      }
      GuidoLabel label1 {
        -label "<html><font color=green>Any zone between 31 and 60</font> will have <font color=green>positive</font> longitude."
      }
      GuidoFiller fill1 {
      }
      GuidoLabel label2 {
        -label "This will often be represented as a West (W) longitude value."
      }
      GuidoFiller fill2 {
      }
      GuidoLabel label3 {
        -label "Example: A longitude of -112 could also be shown as 112W"
      }
      GuidoFiller fill3 {
      }
    }
  }
}

SclCreateGuidoForm formH $convertKMLtoSTR {}
$formH SclRun {}

if {"$_status" != "apply"} {
  puts "Macro Cancelled"
  return
}

set currentDirectory [pwd]
set originalDirectory [file dirname $kmlFile]
set originalFileName [file tail $kmlFile]
file copy -force "$kmlFile.kml" "$currentDirectory/convert.kml"


#Read the lat long from the kml file
set readFile [open "convert.kml" "r"]
set writeFile [open "convert.str" "w"]
puts $writeFile "$originalFileName,,,ssi_styles:styles.ssi"
puts $writeFile "0,0,0,0,0,0,0"
set PointCount 1
set strNum 1
set isItTheCoordinateLine no
while {![eof $readFile]} {
  gets $readFile line
  set trimLine [string trim $line]
  #puts "Line is $line"
  #puts "TrimLine is $trimLine"
  if {"$trimLine" == "<coordinates>"} {
    set isItTheCoordinateLine "yes"
    gets $readFile line
    set trimLine [string trim $line]
  } elseif {"$trimLine" == "</coordinates>"} {
    set isItTheCoordinateLine "no"
    puts $writeFile "0,0,0,0"
    incr strNum
  } elseif {"$trimLine" == "</kml>"} {
    break
  } else {
    continue
  }
  
  if {$isItTheCoordinateLine == "yes"} {
    set coordList [split $trimLine " "]
    #puts "The coordinate list is $coordList"
    foreach point $coordList {
      set data [split $point ","]
      set longitude [string trim [lindex $data 0]]
      #Have to set longitude to an absolute due to a bug in Surpac not allowing negitive longitudes.
      #set longitude [SclExpr ABS(FORMAT($longitude,9))]
      set longitude [SclExpr FORMAT($longitude,12)]
      #puts "Longitude is $longitude"
      set latitude [string trim [lindex $data 1]]
      set latitude [SclExpr FORMAT($latitude,12)]
      set z [string trim [lindex $data 2]]
      puts $writeFile "$strNum,$PointCount,1,0,$longitude,$latitude,$z"
      incr PointCount
    }
  }
}




puts $writeFile "0,0,0,0,END"
close $readFile
close $writeFile
SclPause 0.3

set east_pos "Y"
#Account for negative longitudes
if {$negativeLongitude == "yes"} {
  set east_pos "N"
}

if {$grid == "mga"} {
set status [ SclFunction "LL TO TM" {
  frm00904={
      {
        in_location="convert.str"
        in_id_range=""
        out_location="convert"
        tm_map="Map Grid of Australia 1994 (MGA94)"
        spheroid="GRS 80"
        false_easting="500000"
        false_northing="10000000"
        scale_factor="0.9996"
        zone_width="6"
        z1_cm="-177"
        ll_format="degrees"
        north_pos="Y"
        east_pos="Y"
        lat_deg="D2"
        lng_deg="D1"
        tm_zone="D4"
        northing="Y"
        easting="X"
      }
    }
  }]
} elseif {"$grid" == "amg"} {

set status [ SclFunction "LL TO TM" {
  frm00904={
      {
        in_location="convert.str"
        in_id_range=""
        out_location="convert"
        tm_map="Australian Map Grid"
        spheroid="Australian National Spheroid"
        false_easting="500000"
        false_northing="10000000"
        scale_factor="0.9996"
        zone_width="6"
        z1_cm="-177"
        ll_format="degrees"
        north_pos="Y"
        east_pos="Y"
        lat_deg="D2"
        lng_deg="D1"
        tm_zone="D4"
        northing="Y"
        easting="X"
      }
    }
  }]
} elseif {$grid == "utmNorth"} {
  set status [ SclFunction "LL TO TM" {
    frm00904={
      {
        in_location="convert.str"
        in_id_range=""
        out_location="convert"
        tm_map="UTM"
        spheroid="WGS 84"
        false_easting="500000"
        false_northing="0"
        scale_factor="0.9996"
        zone_width="6"
        z1_cm="-177"
        ll_format="degrees"
        north_pos="Y"
        east_pos="$east_pos"
        lat_deg="D2"
        lng_deg="D1"
        tm_zone="D4"
        northing="Y"
        easting="X"
      }
    }
  }]
} elseif {$grid == "utmSouth"} {
  set status [ SclFunction "LL TO TM" {
    frm00904={
      {
        in_location="convert.str"
        in_id_range=""
        out_location="convert"
        tm_map="UTM"
        spheroid="WGS 84"
        false_easting="500000"
        false_northing="10000000"
        scale_factor="0.9996"
        zone_width="6"
        z1_cm="-177"
        ll_format="degrees"
        north_pos="Y"
        east_pos="$east_pos"
        lat_deg="D2"
        lng_deg="D1"
        tm_zone="D4"
        northing="Y"
        easting="X"
      }
    }
  }]
} else {
  puts "Unknown grid.
  return
}
#Now save the string file back in the original directory and then delete convert.kml and convert.str from the current directory.
file copy -force "convert.str" "$originalDirectory/${originalFileName}.str"

#delete the temp convert files from the working directory
file delete -force "convert.str"
file delete -force "convert.kml"


puts "Finished converting ${originalFileName}.kml to ${originalFileName}.str"