######################################################################
#
# Macro Name    : a_conversion_str_to_kml.tcl
#
# Created by    : Ash Colton from Dassault Systemes.
#
# Created on    : July 2017
#
# Description   : Will take a file in MGA and convert it to a .kml file in Surpac.
#                 The KML file can then be used in Google Earth
#
# Conditions of use : Macro may be used and distributed freely, however may not be modified from it's original version without first contacting Ash Colton on 08 9420 1384.
#                     If modifications or improvements are made to this macro, please email the modified macro to ash.colton@3ds.com
#
######################################################################
#KML Colours
#No Transparency                #40% Transparency
#Red            641400FF        #Red            3c1400FF
#Blue           64FF7800        #Blue           3cFF7800
#Green          6400FF14        #Green          3c00FF14
#Yellow         6414F0FF        #Yellow         3c14F0FF
#Orange         641478FF        #Orange         3c1478FF
#Pink           647800F0        #Pink           3c7800F0
#Dark green     64007814        #Dark green     3c007814
#Light blue     64F0FF14        #Light blue     3cF0FF14
#Brown          64143C8C        #Brown          3c143C8C
#Purple         64784678        #Purple         3c784678
#Black          64000000        #Black          3c000000
#White          64FFFFFF        #White          3cFFFFFF
#Cyan           64A0783C        #Cyan           3cA0783C

set cm "-177"
set polygonHeight "1"
set width 2

proc writeHeader {writeFile description strFile LookAtLat LookAtLong} {
  puts $writeFile "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
  puts $writeFile "<kml xmlns=\"http://www.opengis.net/kml/2.2\">"
  puts $writeFile "  <Document>"
  puts $writeFile "    <name>$strFile</name>"
  puts $writeFile "    <open>1</open>"
  puts $writeFile "    <description>$description</description>"
  puts $writeFile "    <Style id=\"downArrowIcon\">"
  puts $writeFile "      <IconStyle>"
  puts $writeFile "        <Icon>"
  puts $writeFile "          <href>http://maps.google.com/mapfiles/kml/pal4/icon28.png</href>"
  puts $writeFile "        </Icon>"
  puts $writeFile "      </IconStyle>"
  puts $writeFile "    </Style>"
  puts $writeFile "    <Folder>"
  puts $writeFile "      <name>Mine</name>"
  puts $writeFile "      <visibility>1</visibility>"
  puts $writeFile "      <description></description>"
  puts $writeFile "      <LookAt>"
  puts $writeFile "        <longitude>$LookAtLong</longitude>"
  puts $writeFile "        <latitude>$LookAtLat</latitude>"
  puts $writeFile "        <altitude>0</altitude>"
  puts $writeFile "        <heading>0</heading>"
  puts $writeFile "        <tilt>60</tilt>"
  puts $writeFile "        <range>400</range>"
  puts $writeFile "      </LookAt>"
}

proc writeStylePoly {writeFile colour width} {
  puts $writeFile "      <Style id=\"Polygon\">"
  puts $writeFile "        <LineStyle>"
  puts $writeFile "          <color>$colour</color>"
  puts $writeFile "          <width>$width</width>"
  puts $writeFile "        </LineStyle>"
  puts $writeFile "        <PolyStyle>"
  puts $writeFile "          <color>$colour</color>"
  puts $writeFile "        </PolyStyle>"
  puts $writeFile "      </Style>"
}

proc writeStyleLine {writeFile colour width} {
  puts $writeFile "      <Style id=\"Line\">"
  puts $writeFile "        <LineStyle>"
  puts $writeFile "          <color>$colour</color>"
  puts $writeFile "          <width>$width</width>"
  puts $writeFile "        </LineStyle>"
  puts $writeFile "      </Style>"
}

proc writeCoordsLine {writeFile style latLongList} {
  set PointCount 1
  set lastPoint "wasNotASegment"
  foreach point $latLongList {
    set pointData [split $point ","]
    set longitude [string trim [lindex $pointData 0]]
    set latitude [string trim [lindex $pointData 1]]
    set zValue [string trim [lindex $pointData 2]]
    set strNum [string trim [lindex $pointData 3]]
    set point "$longitude,$latitude,$zValue"
    if {"$longitude" == ""} {
      set lastPoint "wasASegment"
      continue
    }
    if {"$PointCount" == "1"} {
      puts $writeFile "      <Placemark>"
      puts $writeFile "        <name>String $strNum</name>"
      puts $writeFile "        <visibility>1</visibility>"
      puts $writeFile "        <styleUrl>#$style</styleUrl>"
      puts $writeFile "        <LineString>"
      puts $writeFile "          <tessellate>1</tessellate>"
      puts $writeFile "          <altitudeMode>relativeToGround</altitudeMode>"
      puts $writeFile "          <coordinates>"
      puts $writeFile "            $point"
      set lastPoint "wasNotASegment"
      incr PointCount
      continue
    } elseif {"$PointCount" > "1" && "$lastPoint" == "wasASegment" } {
      puts $writeFile "          </coordinates>"
      puts $writeFile "    </LineString>"
      puts $writeFile "  </Placemark>"
      puts $writeFile "  <Placemark>"
      puts $writeFile "    <name>String $strNum</name>"
      puts $writeFile "    <visibility>1</visibility>"
      puts $writeFile "    <styleUrl>#$style</styleUrl>"
      puts $writeFile "    <LineString>"
      puts $writeFile "      <tessellate>1</tessellate>"
      puts $writeFile "      <altitudeMode>relativeToGround</altitudeMode>"
      puts $writeFile "        <coordinates>"
      puts $writeFile "          $point"
      set lastPoint "wasNotASegment"
      incr PointCount
      continue
    } elseif {"$lastPoint" == "wasNotASegment"} {
      puts $writeFile "            $point"
      incr PointCount
      continue
    } else {
      puts "Something is messed up"
      return
    }
  }
  puts $writeFile "          </coordinates>"
  puts $writeFile "        </LineString>"
  puts $writeFile "      </Placemark>"
}
proc writeCoordsPoly {writeFile style latLongList} {
  set PointCount 1
  set lastPoint "wasNotASegment"
  foreach point $latLongList {
    set pointData [split $point ","]
    set longitude [string trim [lindex $pointData 0]]
    set latitude [string trim [lindex $pointData 1]]
    set zValue [string trim [lindex $pointData 2]]
    set strNum [string trim [lindex $pointData 3]]
    set point "$longitude,$latitude,$zValue"
    if {"$longitude" == ""} {
      set lastPoint "wasASegment"
      continue
    }
    if {"$PointCount" == "1"} {
      puts $writeFile "      <Placemark>"
      puts $writeFile "        <name>String $strNum</name>"
      puts $writeFile "        <visibility>1</visibility>"
      puts $writeFile "        <styleUrl>#$style</styleUrl>"
      puts $writeFile "        <Polygon>"
      puts $writeFile "          <extrude>1</extrude>"
      puts $writeFile "          <altitudeMode>absolute</altitudeMode>"
      puts $writeFile "          <outerBoundaryIs>"
      puts $writeFile "            <LinearRing>"
      puts $writeFile "              <coordinates>"
      puts $writeFile "                $point"
      set lastPoint "wasNotASegment"
      incr PointCount
      continue
    } elseif {"$PointCount" > "1" && "$lastPoint" == "wasASegment" } {
      puts $writeFile "                </coordinates>"
      puts $writeFile "              </LinearRing>"
      puts $writeFile "            </outerBoundaryIs>"
      puts $writeFile "          </Polygon>"
      puts $writeFile "        </Placemark>"
      puts $writeFile "        <Placemark>"
      puts $writeFile "          <name>String $strNum</name>"
      puts $writeFile "          <visibility>1</visibility>"
      puts $writeFile "          <styleUrl>#$style</styleUrl>"
      puts $writeFile "          <Polygon>"
      puts $writeFile "            <extrude>1</extrude>"
      puts $writeFile "            <altitudeMode>absolute</altitudeMode>"
      puts $writeFile "            <outerBoundaryIs>"
      puts $writeFile "              <LinearRing>"
      puts $writeFile "                <coordinates>"
      puts $writeFile "                  $point"
      set lastPoint "wasNotASegment"
      incr PointCount
      continue
    } elseif {"$lastPoint" == "wasNotASegment"} {
      puts $writeFile "                $point"
      incr PointCount
      continue
    } else {
      puts "Something is messed up"
      return
    }
  }
  puts $writeFile "              </coordinates>"
  puts $writeFile "            </LinearRing>"
  puts $writeFile "          </outerBoundaryIs>"
  puts $writeFile "        </Polygon>"
  puts $writeFile "      </Placemark>"
}

proc writeEnding {writeFile} {
  puts $writeFile "    </Folder>"
  puts $writeFile "  </Document>"
  puts $writeFile "</kml>"
}
 
set convertStrLL {
  GuidoForm form {
    -label "Convert STR to KML"
    -default_buttons
    -defaults_key "MACRO_str_to_kml"
    -layout BoxLayout Y_AXIS

    GuidoPanel panel1 {
      -label "Select File"
      -border etched true
      -layout CentreLineLayout

      GuidoFileBrowserField strFile {
        -label "Select string file"
        -width 30
        -null false
        -extension false
        -absolutePath true
        -file_mask *.str
      }
      GuidoField description {
        -label "Short description"
        -width 15
        -null false
        -format "none"
      }
      GuidoField zone {
        -label "Zone"
        -width 10
        -null false
      }
    }
    GuidoButtonGroupPanel grid {
      -label "Current Grid System"
      -border etched true
      GuidoRadioButton mga {
        -caption MGA
        -selected_value mga
        -tip "UTM North has a false northing of 0m and a false easting of 500000m whereas UTM South has a false northing of 10000000m and a false easting of 500000m. Both UTM North and South use WGS 84 as their spheroid."
      }
#       GuidoRadioButton amg {
#         -caption AMG
#         -selected_value amg
#         -tip "UTM North has a false northing of 0m and a false easting of 500000m whereas UTM South has a false northing of 10000000m and a false easting of 500000m. Both UTM North and South use WGS 84 as their spheroid."
#       }
      GuidoRadioButton utmNorth {
        -caption "North of Equator"
        -selected_value utmNorth
        -tip "UTM North has a false northing of 0m and a false easting of 500000m whereas UTM South has a false northing of 10000000m and a false easting of 500000m. Both UTM North and South use WGS 84 as their spheroid."
      }
      GuidoRadioButton utmSouth {
        -caption "South of Equator"
        -selected_value utmSouth
        -tip "UTM North has a false northing of 0m and a false easting of 500000m whereas UTM South has a false northing of 10000000m and a false easting of 500000m. Both UTM North and South use WGS 84 as their spheroid."
      }
    }
    GuidoPanel panel2 {
      -label "Options"
      -border etched true
      -layout CentreLineLayout

      GuidoComboBox style {
        -label "Select style"
        -value_in "Black Lines" "Blue Lines" "Green Lines" "Red Lines" "Yellow Lines" "Orange Lines" "Dark green Lines" "Light blue Lines" "Brown Lines" "Purple Lines" "Pink Lines" "White Lines" "Cyan Lines" "Blue Polygons" "Green Polygons" "Orange Polygons"
        -exclusive true
        -width 10
        -default "Black Lines"
      }
      GuidoCheckBox flattenFile {
        -label "Flatten Z Values"
        -caption "yes"
        -selected_value yes
        -unselected_value no
      }
    }
  }
}

SclCreateGuidoForm formH $convertStrLL {}
$formH SclRun {}

if {"$_status" != "apply"} {
  puts "Macro Cancelled"
  return
}
set currentDirectory [pwd]
set originalDirectory [file dirname $strFile]
set originalFileName [file tail $strFile]
file copy -force "$strFile.str" "$currentDirectory/convert.str"


#puts $originalDirectory
#puts $originalFileName

#*************************************************************
#KML Colours
#No Transparency                #40% Transparency
#Red            641400FF        #Red            3c1400FF
#Blue           64FF7800        #Blue           3cFF7800
#Green          6400FF14        #Green          3c00FF14
#Yellow         6414F0FF        #Yellow         3c14F0FF
#Orange         641478FF        #Orange         3c1478FF
#Pink           647800F0        #Pink           3c7800F0
#Dark green     64007814        #Dark green     3c007814
#Light blue     64F0FF14        #Light blue     3cF0FF14
#Brown          64143C8C        #Brown          3c143C8C
#Purple         64784678        #Purple         3c784678
#Black          64000000        #Black          3c000000
#White          64FFFFFF        #White          3cFFFFFF
#Cyan           64A0783C        #Cyan           3cA0783C

#styles lookup
if {$style == "Black Lines"} {
  set style "Line"
  set colour "ff000000"
} elseif {$style == "Blue Lines"} {
  set style "Line"
  set colour "ffff7800"
} elseif {$style == "Light blue Lines"} {
  set style "Line"
  set colour "fff0ff14"
} elseif {$style == "Green Lines"} {
  set style "Line"
  set colour "ff00ff14"
} elseif {$style == "Dark green Lines"} {
  set style "Line"
  set colour "ff007814"
} elseif {$style == "Red Lines"} {
  set style "Line"
  set colour "ff1400ff"
} elseif {$style == "Yellow Lines"} {
  set style "Line"
  set colour "ff14F0ff"
} elseif {$style == "Orange Lines"} {
  set style "Line"
  set colour "ff1478ff"
} elseif {$style == "Brown Lines"} {
  set style "Line"
  set colour "ff143C8C"
} elseif {$style == "Purple Lines"} {
  set style "Line"
  set colour "ff784678"
} elseif {$style == "White Lines"} {
  set style "Line"
  set colour "ffffffff"
} elseif {$style == "Cyan Lines"} {
  set style "Line"
  set colour "ffA0783C"
} elseif {$style == "Pink Lines"} {
  set style "Line"
  set colour "ff7800F0"
} elseif {$style == "Blue Polygons"} {
  set style "Polygon"
  set colour "3cFF7800"
} elseif {$style == "Green Polygons"} {
  set style "Polygon"
  set colour "3c00FF14"
} elseif {$style == "Orange Polygons"} {
  set style "Polygon"
  set colour "3c1478FF"
} else {
  puts "Style unknown"
  return
}

proc TM_LatLong {strFile zone cm grid} {
  if {"$grid" == "mga"} {
    set status [ SclFunction "TM TO LL" {
      frm00905={
        {
          in_location="$strFile.str"
          in_id_range=""
          out_location="${strFile}_elev"
          tm_map="Map Grid of Australia 1994 (MGA94)"
          spheroid="GRS 80"
          false_easting="500000"
          false_northing="10000000"
          scale_factor="0.9996"
          zone_width="6"
          z1_cm="$cm"
          ll_format="degrees"
          north_pos="Y"
          east_pos="Y"
          lat_deg="D1"
          lng_deg="D2"
          tm_zone="D1"
          northing="Y"
          easting="X"
          set_zone_val="Y"
          zone_val="$zone"
        }
      }
    }]
  } elseif {"$grid" == "amg"} {
    set status [ SclFunction "TM TO LL" {
      frm00905={
        {
          in_location="$strFile.str"
          in_id_range=""
          out_location="${strFile}_elev"
          tm_map="Australian Map Grid"
          spheroid="Australian National Spheroid"
          false_easting="500000"
          false_northing="10000000"
          scale_factor="0.9996"
          zone_width="6"
          z1_cm="$cm"
          ll_format="degrees"
          north_pos="Y"
          east_pos="Y"
          lat_deg="D1"
          lng_deg="D2"
          tm_zone="D1"
          northing="Y"
          easting="X"
          set_zone_val="Y"
          zone_val="$zone"
        }
      }
    }]
  } elseif {$grid == "utmNorth"} {
    set status [ SclFunction "TM TO LL" {
      frm00905={
        {
          in_location="$strFile.str"
          in_id_range=""
          out_location="${strFile}_elev"
          tm_map="UTM-north"
          spheroid="WGS 84"
          false_easting="500000"
          false_northing="0"
          scale_factor="0.9996"
          zone_width="6"
          z1_cm="$cm"
          ll_format="degrees"
          north_pos="Y"
          east_pos="Y"
          lat_deg="D1"
          lng_deg="D2"
          tm_zone="D4"
          northing="Y"
          easting="X"
          set_zone_val="Y"
          zone_val="$zone"
        }
      }
    }]
  } elseif {$grid == "utmSouth"} {
    set status [ SclFunction "TM TO LL" {
      frm00905={
        {
          in_location="$strFile.str"
          in_id_range=""
          out_location="${strFile}_elev"
          tm_map="UTM-south"
          spheroid="WGS 84"
          false_easting="500000"
          false_northing="10000000"
          scale_factor="0.9996"
          zone_width="6"
          z1_cm="$cm"
          ll_format="degrees"
          north_pos="Y"
          east_pos="Y"
          lat_deg="D1"
          lng_deg="D2"
          tm_zone="D4"
          northing="Y"
          easting="X"
          set_zone_val="Y"
          zone_val="$zone"
        }
      }
    }]
  } else {
    puts "Don't know what happened"
    return
  }
}

TM_LatLong "convert" "$zone" "$cm" "$grid"

#****************************************************************************
if {$flattenFile == "no"} {
  set readFile [open "convert.str" "r"]
  gets $readFile
  gets $readFile
  
  set minZ 1000000000000
  while {![eof $readFile]} {
    gets $readFile line
    set data [split $line ","]
    set strNum [string trim [lindex $data 0]]
    set zValue [string trim [lindex $data 3]]
    if {"$strNum" == "0"} {
      continue
    }
    if {"$zValue" < "$minZ" && "$zValue" != ""} {
      set minZ "$zValue"
      #puts $minZ
    }
  }
  close $readFile
  #puts "Minimum Z value is $minZ"
}

#Read the lat long from the converted file
set readFile [open "convert_elev.str" "r"]
set writeFile [open "convert.kml" "w"]
gets $readFile
gets $readFile
set PointCount 1
while {![eof $readFile]} {
  gets $readFile line
  set data [split $line ","]
  set strNum [string trim [lindex $data 0]]
  set zValue [string trim [lindex $data 3]]
  #puts "Z value is $zValue"
  if {"$flattenFile" == "no"} {
    set zValue [expr $zValue-$minZ]
  } elseif {"$flattenFile" == "yes"} {
    set zValue 1
  } else {
    puts "Unknown Z value"
  }
  set long [string trim [lindex $data 5]]
  set lat [string trim [lindex $data 4]]

  lappend latLongList "$long,$lat,$zValue,$strNum"
  if {"$PointCount" == "1"} {
    set LookAtLat $lat
    set LookAtLong $long
  }
  incr PointCount
}
close $readFile
#puts "latLongList is $latLongList"

#Conversion of files complete.  Now create the kml file for Google earth



writeHeader $writeFile $description $originalFileName $LookAtLat $LookAtLong
if {"$style" == "Line"} {
  writeStyleLine $writeFile $colour $width
  writeCoordsLine $writeFile $style $latLongList
} elseif {"$style" == "Polygon"} {
  writeStylePoly $writeFile $colour $width
  writeCoordsPoly $writeFile $style $latLongList
} else {
  puts "Something has gone wrong"
  return
}
writeEnding $writeFile

close $writeFile
#Now rename convert.str, convert_elev.str and convert.kml to the original file names
#file copy -force "convert_elev.str" "$originalDirectory/${originalFileName}_elev.str"
file copy -force "convert.kml" "$originalDirectory/$originalFileName.kml"
#delete the temp convert files from the working directory
file delete -force "convert.str"
file delete -force "convert_elev.str"
file delete -force "convert.kml"

puts "${strFile}.kml file has been written.  Use File Open in Google Earth to see the result."
#puts "${strFile}_elev.str file has also been written.  This can be used to retrieve altitude from Google servers but requires the use of an API key which DS cannot provide.  Check Google website for more information."


